@@ -1,5 +1,5 @@
 /*
-Copyright 2019 Ron Triepels
+Copyright 2020 Ron Triepels
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -21,13 +21,6 @@ limitations under the License.
 
 #include "class.h"
 
-/*
- * SYMBOLS
- */
-
-#define CG_SESSION_SYMBOL Rf_install("session")
-#define CG_GRAPH_SYMBOL Rf_install("graph")
-
 /*
  * PRIVATE METHODS
  */
@@ -56,7 +49,7 @@ SEXP cg_session_graph()
 {
   SEXP session = PROTECT(cg_session_get());
 
-  SEXP graph = PROTECT(Rf_findVarInFrame(session, CG_GRAPH_SYMBOL));
+  SEXP graph = PROTECT(CG_GET(session, CG_GRAPH_SYMBOL));
 
   if(graph == R_UnboundValue)
   {
@@ -77,7 +70,7 @@ SEXP cg_session_set_graph(SEXP graph)
 
   SEXP session = PROTECT(cg_session_get());
 
-  Rf_defineVar(CG_GRAPH_SYMBOL, graph, session);
+  CG_SET(session, CG_GRAPH_SYMBOL, graph);
 
   UNPROTECT(1);
 
@@ -94,7 +87,7 @@ SEXP cg_session()
 
   SEXP env = PROTECT(R_FindNamespace(Rf_mkString("cgraph")));
 
-  Rf_defineVar(CG_SESSION_SYMBOL, session, env);
+  CG_SET(env, CG_SESSION_SYMBOL, session);
 
   UNPROTECT(2);
 
